const influx = require("influx");

const inflog = {

    /**
     * inflog.createInstance()
     * @returns {boolean|Promise}
     */
    createInstance() {

        if (process.env.INFLOG !== "true")

            return false;

        return new Promise((resolve, reject) => {

            const influxObject = new influx.InfluxDB({
                host: process.env.INFLOG_HOST,
                port: process.env.INFLOG_PORT,
                database: process.env.INFLOG_DATABASE
            });

            influxObject.getDatabaseNames()

                .then(async (nameObjects) => {

                    if (nameObjects.includes(process.env.INFLOG_DATABASE) === false) {

                        await influxObject.createDatabase(process.env.INFLOG_DATABASE);

                        resolve(influxObject);

                    }

                    resolve(influxObject);

                })

                .catch((error) => {

                    reject(error);

                });

        });

    },

    /**
     * inflog.createRecord()
     * @param {string} measurement
     * @param {object} requestTags
     * @param {object} requestFields
     * @returns {boolean}
     */
    createRecord(measurement, requestTags, requestFields) {

        console.log(measurement, requestTags, requestFields);

        // Step #1
        // Make sure the INFLOG is enabled, if not, just print it out.

        if (process.env.INFLOG !== "true")

            return false;

        // Step #2
        // Create a new Influx connection

        let influxObject = new influx.InfluxDB({
            host: process.env.INFLOG_HOST,
            port: process.env.INFLOG_PORT,
            database: process.env.INFLOG_DATABASE
        });

        // Step #3
        // Storing the record into Influx

        influxObject.writePoints([{
            measurement: measurement,
            tags: requestTags,
            fields: requestFields,
        }]);

        return true;

    },

    /**
     * inflog.error()
     * @param {string} requestSummary
     * @param {object} requestDescription
     */
    error(requestSummary, requestDescription = {}) {

        let requestTags = {
            "instance": process.env.INFLOG_INSTANCE,
            "level": "ERROR"
        };

        let requestFields = Object.assign(requestDescription, {
            "summary": requestSummary
        });

        inflog.createRecord("service_log", requestTags, requestFields);

    },

    /**
     * inflog.warning()
     * @param {string} requestSummary
     * @param {object} requestDescription
     */
    warning(requestSummary, requestDescription = {}) {

        let requestTags = {
            "instance": process.env.INFLOG_INSTANCE,
            "level": "WARNING"
        };

        let requestFields = Object.assign(requestDescription, {
            "summary": requestSummary
        });

        inflog.createRecord("service_log", requestTags, requestFields);

    },

    /**
     * inflog.info()
     * @param {string} requestSummary
     * @param {object} requestDescription
     */
    info(requestSummary, requestDescription = {}) {

        let requestTags = {
            "instance": process.env.INFLOG_INSTANCE,
            "level": "INFO"
        };

        let requestFields = Object.assign(requestDescription, {
            "summary": requestSummary
        });

        inflog.createRecord("service_log", requestTags, requestFields);

    },

    /**
     * inflog.action()
     * @param {string} requestName
     * @param {object} requestFields
     */
    action(requestName, requestFields = {}) {

        let requestTags = {
            "instance": process.env.INFLOG_INSTANCE,
            "action": requestName
        };

        inflog.createRecord("service_action", requestTags, requestFields);

    },

    /**
     * inflog.response
     * @param {string} inputMethod
     * @param {string} inputPath
     * @param {int} inputStatus
     * @param {int} inputDuration
     */
    response(inputMethod, inputPath, inputStatus, inputDuration) {

        let requestTags = {
            "instance": process.env.INFLOG_INSTANCE
        };

        let requestFields = {
            "method": inputMethod,
            "path": inputPath,
            "status": inputStatus,
            "duration": inputDuration,
        };

        inflog.createRecord("service_response", requestTags, requestFields);

    }

};

module.exports = inflog;